"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _react = require("react");

var _react2 = _interopRequireDefault(_react);

var _FormContext = require("./FormContext");

var _FormContext2 = _interopRequireDefault(_FormContext);

var _FormFragment = require("./FormFragment");

var _FormFragment2 = _interopRequireDefault(_FormFragment);

var _DefaultField = require("./DefaultField");

var _DefaultField2 = _interopRequireDefault(_DefaultField);

var _utils = require("../utilities/utils");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var Form = function (_Component) {
  _inherits(Form, _Component);

  function Form(props) {
    _classCallCheck(this, Form);

    var _this = _possibleConstructorReturn(this, (Form.__proto__ || Object.getPrototypeOf(Form)).call(this, props));

    _this.state = {
      fields: [],
      value: {},
      isValid: false,
      defaultFields: []
    };
    return _this;
  }

  _createClass(Form, [{
    key: "onFieldChange",
    value: function onFieldChange(id, value) {
      var _this2 = this;

      var _props = this.props,
          optionsHandler = _props.optionsHandler,
          parentContext = _props.parentContext;
      var fields = this.state.fields;

      fields = (0, _utils.updateFieldValue)(id, value, fields);
      var nextState = (0, _utils.getNextStateFromFields)(fields, optionsHandler, parentContext);

      this.setState(function (state, props) {
        return nextState;
      }, function () {
        var onChange = _this2.props.onChange;
        var value = nextState.value,
            isValid = nextState.isValid;

        if (onChange) {
          onChange(value, isValid);
        }
      });
    }

    // Register field is provided in the context to allow children to register with this form...

  }, {
    key: "registerField",
    value: function registerField(field) {
      var _state = this.state,
          _state$fields = _state.fields,
          fields = _state$fields === undefined ? [] : _state$fields,
          _state$value = _state.value,
          value = _state$value === undefined ? {} : _state$value;


      if (fields.find(function (existingField) {
        return field.id === existingField.id;
      })) {
        // Don't register fields twice...
        // console.warn("Field ID already in use", field.id);
      } else {
        fields = (0, _utils.registerField)(field, fields, value);
        this.setState(function (state, props) {
          var filteredFields = state.fields.filter(function (existingField) {
            return existingField.id !== field.id;
          });
          // let updatedFields = fields.concat(filteredFields);
          var updatedFields = filteredFields.concat(field);
          var nextState = (0, _utils.getNextStateFromFields)(updatedFields, props.optionsHandler, props.parentContext);
          return _extends({}, nextState);
        });
      }
    }
  }, {
    key: "createFormContext",
    value: function createFormContext() {
      var _state2 = this.state,
          fields = _state2.fields,
          value = _state2.value,
          isValid = _state2.isValid;
      var _props2 = this.props,
          _props2$renderField = _props2.renderField,
          renderField = _props2$renderField === undefined ? this.renderField : _props2$renderField,
          optionsHandler = _props2.optionsHandler,
          parentContext = _props2.parentContext;

      var onFieldChange = this.onFieldChange.bind(this);

      var context = {
        fields: fields,
        isValid: isValid,
        value: value,
        registerField: this.registerField.bind(this),
        renderField: renderField,
        optionsHandler: optionsHandler,
        options: {},
        onFieldChange: onFieldChange,
        parentContext: parentContext
      };

      return context;
    }
  }, {
    key: "renderField",
    value: function renderField(field, onChange) {
      var visible = field.visible;

      if (!visible) {
        return null;
      }
      return _react2.default.createElement(_DefaultField2.default, { key: field.id, field: field, onChange: onChange });
    }
  }, {
    key: "renderFields",
    value: function renderFields(context) {
      var fields = context.fields,
          onFieldChange = context.onFieldChange,
          renderField = context.renderField;

      var renderedFields = fields.map(function (field) {
        var visible = field.visible;

        if (visible) {
          return renderField(field, onFieldChange);
        }
        return null;
      });

      return _react2.default.createElement(
        _FormContext2.default.Provider,
        { value: context },
        renderedFields
      );
    }
  }, {
    key: "render",
    value: function render() {
      var _props3 = this.props,
          children = _props3.children,
          defaultFields = _props3.defaultFields;
      var fields = this.state.fields;

      var context = this.createFormContext();
      return _react2.default.createElement(
        _FormContext2.default.Provider,
        { value: context },
        defaultFields && _react2.default.createElement(_FormFragment2.default, { defaultFields: fields }),
        children
      );
    }
  }], [{
    key: "getDerivedStateFromProps",
    value: function getDerivedStateFromProps(nextProps, prevState) {
      if (nextProps.defaultFields && nextProps.defaultFields !== prevState.defaultFields) {
        var valueFromState = prevState.value;
        var defaultFields = nextProps.defaultFields,
            valueFromProps = nextProps.value;
        var optionsHandler = nextProps.optionsHandler,
            parentContext = nextProps.parentContext;

        var fields = (0, _utils.registerFields)(defaultFields, valueFromProps || valueFromState || {});
        var nextState = (0, _utils.getNextStateFromFields)(fields, optionsHandler, parentContext);

        return _extends({}, nextState, {
          defaultFields: defaultFields
        });
      } else {
        return null;
      }
    }
  }]);

  return Form;
}(_react.Component);

exports.default = Form;