'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
var lengthIsGreaterThan = exports.lengthIsGreaterThan = function lengthIsGreaterThan(_ref) {
  var value = _ref.value,
      length = _ref.length,
      message = _ref.message;

  if (isNaN(length) || (value || '').length > length) {
    return;
  } else {
    return message || 'Should have more than ' + length + ' characters';
  }
};
var lengthIsLessThan = exports.lengthIsLessThan = function lengthIsLessThan(_ref2) {
  var value = _ref2.value,
      length = _ref2.length,
      message = _ref2.message;

  if (isNaN(length) || (value || '').length < length) {
    return;
  } else {
    return message || 'Should have more than ' + length + ' characters';
  }
};

// TODO: Consider option for inverting rule...
var matchesRegEx = exports.matchesRegEx = function matchesRegEx(_ref3) {
  var value = _ref3.value,
      _ref3$pattern = _ref3.pattern,
      pattern = _ref3$pattern === undefined ? '.*' : _ref3$pattern,
      message = _ref3.message;

  var regExObj = new RegExp(pattern);
  if (!regExObj.test(value)) {
    return message || 'Invalid input provided'; // <= Terrible message!
  }
};

var getDefaultNumericalRangeErrorMessages = exports.getDefaultNumericalRangeErrorMessages = function getDefaultNumericalRangeErrorMessages(min, max) {
  if (typeof min !== 'undefined' && typeof max !== 'undefined') {
    return 'Value cannot be less than ' + min + ' or greater than ' + max;
  } else if (typeof min !== 'undefined') {
    return 'Value cannot be less than ' + min;
  } else if (typeof max !== 'undefined') {
    return 'Value cannot be greater than ' + max;
  }
};

var fallsWithinNumericalRange = exports.fallsWithinNumericalRange = function fallsWithinNumericalRange(_ref4) {
  var value = _ref4.value,
      min = _ref4.min,
      max = _ref4.max,
      required = _ref4.required,
      message = _ref4.message;

  var parsedValue = parseFloat(value);
  if (isNaN(parsedValue)) {
    return message || 'Value must be a number';
  }
  if (typeof min !== 'undefined' && value < min) {
    return message || getDefaultNumericalRangeErrorMessages(min, max);
  }
  if (typeof max !== 'undefined' && value > max) {
    return message || getDefaultNumericalRangeErrorMessages(min, max);
  }
};

var validators = exports.validators = {
  lengthIsGreaterThan: lengthIsGreaterThan,
  lengthIsLessThan: lengthIsLessThan,
  matchesRegEx: matchesRegEx,
  fallsWithinNumericalRange: fallsWithinNumericalRange
};

var validateField = exports.validateField = function validateField(field) {
  var required = field.required,
      visible = field.visible,
      value = field.value,
      _field$validWhen = field.validWhen,
      validWhen = _field$validWhen === undefined ? {} : _field$validWhen;

  var isValid = true;
  var errorMessages = [];
  if (visible) {
    if (required) {
      var valueIsEmptyArray = Array.isArray(value) && value.length === 0;
      isValid = (value || value === 0 || value === false) && !valueIsEmptyArray;
    }

    isValid = Object.keys(validWhen).reduce(function (allValidatorsPass, validator) {
      if (typeof validators[validator] === 'function') {
        // $FlowFixMe
        var validationConfig = validWhen[validator];
        validationConfig.value = value;
        var _message = validators[validator](validationConfig);
        if (_message) {
          allValidatorsPass = false;
          errorMessages.push(_message);
        }
      } else {
        console.warn('The requested validator does not exist', validator);
      }
      return allValidatorsPass;
    }, isValid) && isValid;
  }
  return Object.assign({}, field, {
    isValid: isValid,
    errorMessages: errorMessages.join(', ')
  });
};

var validateAllFields = exports.validateAllFields = function validateAllFields(fields) {
  var validatedFields = fields.map(function (field) {
    return validateField(field);
  });
  return validatedFields;
};