'use strict';

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _validation = require('./validation');

var _utils = require('./utils.js');

var field1 = (0, _utils.createField)({
  id: 'one',
  name: 'name',
  value: 'value'
});

describe('validateField', function () {
  test('visible, optional field is always valid', function () {
    var testField = _extends({}, field1, {
      visible: true,
      required: false
    });
    expect((0, _validation.validateField)(testField).isValid).toBe(true);
  });

  test('visible, required field with empty string value is valid', function () {
    var testField = _extends({}, field1, {
      visible: true,
      required: true,
      value: ''
    });
    expect((0, _validation.validateField)(testField).isValid).toBe(false);
  });

  test('visible, required field with numberical value 0 is valid', function () {
    var testField = _extends({}, field1, {
      visible: true,
      required: true,
      value: 0
    });
    expect((0, _validation.validateField)(testField).isValid).toBe(true);
  });

  test('visible, required field with false value is valid', function () {
    var testField = _extends({}, field1, {
      visible: true,
      required: true,
      value: false
    });
    expect((0, _validation.validateField)(testField).isValid).toBe(true);
  });

  test('visible, required field with string value is valid', function () {
    var testField = _extends({}, field1, {
      visible: true,
      required: true,
      value: 'test'
    });
    expect((0, _validation.validateField)(testField).isValid).toBe(true);
  });

  test('visible, required field with empty array value is valid', function () {
    var testField = _extends({}, field1, {
      visible: true,
      required: true,
      value: []
    });
    expect((0, _validation.validateField)(testField).isValid).toBe(false);
  });

  test('visible, required field with populated array value is valid', function () {
    var testField = _extends({}, field1, {
      visible: true,
      required: true,
      value: [1]
    });
    expect((0, _validation.validateField)(testField).isValid).toBe(true);
  });
});

describe('lengthIsGreaterThan validator', function () {
  test('with valid value', function () {
    expect((0, _validation.lengthIsGreaterThan)({
      value: 'test',
      length: 3,
      message: 'Fail'
    })).toBeUndefined();
  });

  test('with invvalid value', function () {
    expect((0, _validation.lengthIsGreaterThan)({
      value: 'te',
      length: 3,
      message: 'Fail'
    })).toBe('Fail');
  });
});

describe('lengthIsLessThan validator', function () {
  test('with valid value', function () {
    expect((0, _validation.lengthIsLessThan)({
      value: 'te',
      length: 3,
      message: 'Fail'
    })).toBeUndefined();
  });

  test('with invvalid value', function () {
    expect((0, _validation.lengthIsLessThan)({
      value: 'test',
      length: 3,
      message: 'Fail'
    })).toBe('Fail');
  });
});

describe('matchesRegEx validator', function () {
  test('fails when letters provided for numbers only pattern', function () {
    expect((0, _validation.matchesRegEx)({ value: '12a3', pattern: '^[\\d]+$', message: 'Fail' })).toBe('Fail');
  });

  test('succeeds when numbers provided for numbers only pattern', function () {
    expect((0, _validation.matchesRegEx)({ value: '1234', pattern: '^[\\d]+$', message: 'Fail' })).toBeUndefined();
  });
});

describe('getDefaultNumericalRangeErrorMessages', function () {
  test('for min and max', function () {
    expect((0, _validation.getDefaultNumericalRangeErrorMessages)(1, 5)).toEqual('Value cannot be less than 1 or greater than 5');
  });
  test('for just min', function () {
    expect((0, _validation.getDefaultNumericalRangeErrorMessages)(1)).toEqual('Value cannot be less than 1');
  });
  test('for just max', function () {
    expect((0, _validation.getDefaultNumericalRangeErrorMessages)(undefined, 5)).toEqual('Value cannot be greater than 5');
  });
});

describe('fallsWithinNumericalRange', function () {
  test('fails when given a non-numerical number', function () {
    expect((0, _validation.fallsWithinNumericalRange)({ value: 'abc', min: 5, message: 'Fail' })).toBe('Fail');
  });

  test('succeeds with just a min', function () {
    expect((0, _validation.fallsWithinNumericalRange)({
      value: '5',
      min: 1,
      message: 'Fail'
    })).toBeUndefined();
  });

  test('succeeds with just a max', function () {
    expect((0, _validation.fallsWithinNumericalRange)({
      value: 6,
      max: 10,
      message: 'Fail'
    })).toBeUndefined();
  });

  test('fails with just a min', function () {
    expect((0, _validation.fallsWithinNumericalRange)({
      value: '5',
      min: 10,
      message: 'Fail'
    })).toBe('Fail');
  });

  test('fails with just a max', function () {
    expect((0, _validation.fallsWithinNumericalRange)({
      value: 6,
      max: 5,
      message: 'Fail'
    })).toBe('Fail');
  });
});